import api.util.ISO8601;
import api.http.WebRequest;
import api.http.WebResponse;
import org.apache.commons.lang3.StringUtils;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	if (!StringUtils.equalsIgnoreCase(request.getMethod(), "POST")) {
		// Method Not Allowed
		response.setStatus(405);
		return;
	}

	if (!StringUtils.startsWith(request.getContentType(), "application/json")) {
		// Bad Request
		response.setStatus(400);
		return;
	}

	def req = WebRequest.create(request).parseRequest();
	def isValid = {->
    	if (!req) {
    		return false;
    	}

    	req.businessKey = req.businessKey?.trim();
    	if (!req.businessKey) {
    		return false;
    	}

    	if (req.files == null || req.files.size() == 0) {
    		return false;
    	}

		return true;
	}();
	if (!isValid) {
		// Bad Request
		response.setStatus(400);
		return;
	}

	try {
		def resp = [
			"businessKey": req.businessKey,
			"files": []
		];

		def folder = repositorySession.resourceResolver.getResource("/var/approvals/" + req.businessKey);
		if (folder.exists()) {
			// Conflict
			response.setStatus(409);
			return;
		}

		// create folder
		folder = folder.createFolder();

		// create files
		for (file in req.files) {
			def r = folder.createFile(file.name);

			// set MIME type
			def mimeType = file.type?.trim();
			if (!mimeType) {
				def type = MimeTypeAPI.getMimeType(file.name);
				if (type) {
					mimeType = type;
				}
			}
			if (mimeType) {
				r.setProperty("jcr:mimeType", mimeType);
			}

			// write content
			def data = file.data?.trim();
			def p = data.indexOf(";base64,");
			if (p == -1) {
				// Bad Request
				response.setStatus(400);
				return;
			}
			r.write(data.substring(p + 8).decodeBase64());

			// set modified
			r.setProperty("jcr:lastModified", ISO8601.parseDate(file.modified));

			resp.files.add([
				"name": file.name,
				"type": file.type,
				"size": file.size,
				"modified": file.modified
			]);
		}

		// commit
		repositorySession.commit();

		// OK
		WebResponse
		    .create(response)
    		.setStatus(200)
    		.setContentType("application/json")
		    .writeAsJson(resp);
	} catch (Throwable ex) {
	    response.sendError(500, ex.message);
		return;
	}
}();
